///////////////////////////////////////////////////////////////
// DEFINE
///////////////////////////////////////////////////////////////
#define DF_TIMEOUT					5		// $B%?%$%`%"%&%H;~4V(B(0.6$BICC10L(B)
//#define DF_DEBUG_i915						// $B%G%P%C%0%7%s%\%k(B

///////////////////////////////////////////////////////////////
// INCLUDE
///////////////////////////////////////////////////////////////
#include <linux/module.h>			// THIS_MODULE
#include <linux/ioport.h>			// request_region()
#include <asm/io.h>					// inb_p(), outb()
#include <stdbool.h>					// bool
#include <linux/fs.h>				// struct inode
#include <linux/miscdevice.h>		// misc_register()
#include <linux/watchdog.h>			// WATCHDOG_MINOR
#include <linux/notifier.h>			// NOTIFY_DONE
#include <linux/reboot.h>			// register_reboot_notifier()
#include <linux/pci.h>				// struct pci_dev
#include <asm/uaccess.h>				// copy_from_user()
#include <asm/bitops.h>				// test_and_set_bit(), clear_bit()

///////////////////////////////////////////////////////////////
// PROTOTYPE
///////////////////////////////////////////////////////////////
static int __init		i915_watchdog_init(void);									// $B%I%i%$%P=i4|2==hM}(B
static void __exit		i915_watchdog_cleanup(void);								// $B%I%i%$%P%j%j!<%9=hM}(B

static int				i915_watchdog_open(struct inode*,struct file*);			// open()$B%7%9%F%`%3!<%k(B
static int				i915_watchdog_release(struct inode*,struct file*);		// close()$B%7%9%F%`%3!<%k(B
static int				i915_watchdog_write(struct file*,
												const char __user *, size_t,loff_t *);	// write()$B%7%9%F%`%3!<%k(B
static int				i915_watchdog_ioctl(struct inode*,struct file*,
												unsigned int,unsigned long);			// ioctl()$B%7%9%F%`%3!<%k(B

static bool				i915_watchdog_find(void);									// PCI$B%G%P%$%98!:w=hM}(B
static bool				i915_watchdog_ACPIBASE(void);								// ACPIBASE$B%"%I%l%9<hF@(B
static bool				i915_watchdog_NO_REBOOT(void);								// NO_REBOOT$B%S%C%H%/%j%"(B
static bool				i915_set_timer(unsigned short);								// $B%?%$%`%"%&%H;~4V@_Dj(B
static void				i915_tco_stop(void);											// $B%?%$%^Dd;_(B
static void				i915_tco_reload(void);										// $B%?%$%^$N%j%m!<%I(B
static void				i915_tco_start(void);										// $B%?%$%^3+;O(B
static int				i915_reboot(struct notifier_block*,unsigned long, void*);// $B%j%V!<%H;~4X?t(B

///////////////////////////////////////////////////////////////
// GLOBAL
///////////////////////////////////////////////////////////////
static struct pci_dev *			G_pcidev;							// PCI$B%G%P%$%9>pJs(B
static volatile unsigned int	G_acpibase;						// ACPIBASE$B%"%I%l%9(B

static struct file_operations	G_fops = 							// $B%7%9%F%`%3!<%k$H$N@\B3(B
{
	.owner = THIS_MODULE,
	.llseek = no_llseek,
	.write = i915_watchdog_write,
	.ioctl = i915_watchdog_ioctl,
	.open = i915_watchdog_open,
	.release = i915_watchdog_release
};

static struct miscdevice		G_miscdev = 						// MISC$B%G%P%$%9<1JL>pJs(B
{
	.minor = WATCHDOG_MINOR,
	.name = "watchdog",
	.fops = &G_fops
};

spinlock_t						G_spinlock;						// $BGSB>@)8fMQ(B
unsigned long						G_openFlag;						// $B%*!<%W%s@)8fMQ(B
bool								G_closeReq;						// $B%/%m!<%:M=9p(B
struct notifier_block			G_reboot = 						// $B%j%V!<%H@)8fMQ(B
{
	.notifier_call = i915_reboot,									// $B%j%V!<%H;~<B9T4X?tEPO?(B
};

module_init(i915_watchdog_init);
module_exit(i915_watchdog_cleanup);
//MODULE_ALIAS_MISCDEV(WATCHDOG_MINOR);
MODULE_LICENSE("GPL");
///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////
// FUNCTION: i915_watchdog_init
// NOTE: $B%I%i%$%P=i4|2==hM}(B
///////////////////////////////////////////////////////////////
static int __init i915_watchdog_init(void)
{
	int						wk_ret = -EINVAL;										// $BLa$jCM(B
	volatile unsigned char	wk_reg;													// $B%l%8%9%?A`:nMQ(B

#ifdef DF_DEBUG_i915
	volatile unsigned short wk_sreg;													// $B%G%P%C%0MQ(B
	printk(KERN_INFO "i915_tco: i915_watchdog_init() have been executed.\n");		// $B%G%P%C%0%a%C%;!<%8(B
#endif

	do
	{
		spin_lock_init(&G_spinlock);													// $BGSB>@)8fMQJQ?t=i4|2=(B
		if(i915_watchdog_find() == false)												// PCI$B%G%P%$%9H/8+=PMh$:(B?
		{
			printk(KERN_INFO "i915_tco: PCI device can not be found.\n");			// $B%(%i!<%a%C%;!<%8(B
			break;																			// $B0[>o=*N;(B
		}

		if(i915_watchdog_ACPIBASE() == false)											// ACPIBASE$B<hF@=PMh$:(B?
		{
			printk(KERN_INFO "i915_tco: ACPIBASE can not be got.\n");				// $B%(%i!<%a%C%;!<%8(B
			break;																			// $B0[>o=*N;(B
		}

		if(request_region(G_acpibase+0x31,0x01,"i915_tco") == 0)						// SMI_EN$B%l%8%9%?>e0L%P%$%H<hF@!"<:GT(B?
		{
			printk(KERN_INFO "i915_tco: SMI_EN can not be got.\n");					// $B%(%i!<%a%C%;!<%8(B
			break;																			// $B0[>o=*N;(B
		}
		wk_reg = inb(G_acpibase+0x31);													// SMI_EN$B%l%8%9%?FbMFFI=P$7(B
		wk_reg &= 0xDF;																	// TCO$B%?%$%`%"%&%H$K$h$k(BSMI$BH/9T6X;_(B
		outb(wk_reg,G_acpibase+0x31);													// SMI_EN$B%l%8%9%?$K=q9~$_(B
#ifdef DF_DEBUG_i915
		wk_reg = inb(G_acpibase+0x31);													// SMI_EN$B%l%8%9%?FbMFFI=P$7(B
		printk(KERN_INFO "i915_tco: SMI_EN = [%X]\n",wk_reg);						// $B%G%P%C%0%a%C%;!<%8(B
#endif
		release_region(G_acpibase+0x31,1);												// SMI_EN$B%l%8%9%?3+J|(B

		if(request_region(G_acpibase+0x60,0x14,"i915_tco") == 0)						// TCO$B%l%8%9%?NN0h3NJ]!"<:GT(B?
		{
			printk(KERN_INFO "i915_tco: TCO register can not be got.\n");			// $B%(%i!<%a%C%;!<%8(B
			break;																			// $B0[>o=*N;(B
		}

		i915_tco_stop();																	// $B%?%$%^Dd;_(B
		if(i915_set_timer(DF_TIMEOUT) == false)										// $B%G%U%)%k%H%?%$%`%"%&%H;~4V@_Dj!"<:GT(B?
		{
			printk(KERN_INFO "i915_tco: The timeout value can not be set.\n");		// $B%(%i!<%a%C%;!<%8(B
			release_region(G_acpibase+0x60,0x14);										// TCO$B%l%8%9%?NN0h3+J|(B
			break;																			// $B0[>o=*N;(B
		}

		i915_tco_reload();																// $B%?%$%^%j%m!<%I(B
		outw(0x178E,G_acpibase+0x60+0x04);												// TCO1_STS$B%/%j%"(B
		outw(0x0007,G_acpibase+0x60+0x06);												// TCO2_STS$B%/%j%"(B
#ifdef DF_DEBUG_i915
		wk_sreg = inw(G_acpibase+0x60+0x00);											// TCO_RLD$BFI=P$7(B
		printk(KERN_INFO "i915_tco: TCO1_RLD = [%X]\n",wk_sreg);						// $B%G%P%C%0%a%C%;!<%8(B
		wk_sreg = inw(G_acpibase+0x60+0x04);											// TCO1_STS$BFI=P$7(B
		printk(KERN_INFO "i915_tco: TCO1_STS = [%X]\n",wk_sreg);						// $B%G%P%C%0%a%C%;!<%8(B
		wk_sreg = inw(G_acpibase+0x60+0x06);											// TCO2_STS$BFI=P$7(B
		printk(KERN_INFO "i915_tco: TCO2_STS = [%X]\n",wk_sreg);						// $B%G%P%C%0%a%C%;!<%8(B
#endif

		if(i915_watchdog_NO_REBOOT() == false)											// NO_REBOOT$B%/%j%"=PMh$:(B?
		{
			printk(KERN_INFO "i915_tco: NO_REBOOT can not be cleared.\n");			// $B%(%i!<%a%C%;!<%8(B
			release_region(G_acpibase+0x60,0x14);										// TCO$B%l%8%9%?NN0h3+J|(B
			break;																			// $B0[>o=*N;(B
		}

		if(misc_register(&G_miscdev) != 0)												// MISC$B%G%P%$%9<1JLHV9f<hF@!"<:GT(B?
		{
			printk(KERN_INFO "i915_tco: misc_register() is fail.\n");				// $B%(%i!<%a%C%;!<%8(B
			release_region(G_acpibase+0x60,0x14);										// TCO$B%l%8%9%?NN0h3+J|(B
			break;																			// $B0[>o=*N;(B
		}

		(void)register_reboot_notifier(&G_reboot);									// $B%j%V!<%H4X?tEPO?(B
		G_openFlag = 0;																	// $B%*!<%W%s:Q%U%i%0%/%j%"(B
		G_closeReq = false;																// $B%/%m!<%:M=9p%/%j%"(B
		wk_ret = 0;																		// $BLa$jCM$r@5>o$KJQ99(B

#ifdef DF_DEBUG_i915
		printk(KERN_INFO "i915_tco: G_openFlag = %ld\n",G_openFlag);					// $B%G%P%C%0%a%C%;!<%8(B
		if(G_closeReq == false) printk(KERN_INFO "i915_tco: G_closeReq is ok\n");	// $B%G%P%C%0%a%C%;!<%8(B
		printk(KERN_INFO "i915_tco: i915_watchdog_init() succeeded.\n");			// $B%G%P%C%0%a%C%;!<%8(B
#endif
	}while(0);
	return wk_ret;
}

///////////////////////////////////////////////////////////////
// FUNCTION: i915_watchdog_cleanup
// NOTE: $B%I%i%$%P%j%j!<%9=hM}(B
///////////////////////////////////////////////////////////////
static void __exit i915_watchdog_cleanup(void)
{
#ifdef DF_DEBUG_i915
	printk(KERN_INFO "i915_tco: i915_watchdog_cleanup() have been executed.\n");		// $B%G%P%C%0%a%C%;!<%8(B
#endif

	(void)unregister_reboot_notifier(&G_reboot);											// $B%j%V!<%H4X?tEPO?2r=|(B
	misc_deregister(&G_miscdev);															// $B%G%P%$%9<1JLHV9f3+J|(B
	release_region(G_acpibase+0x60,0x14);													// TCO$B%l%8%9%?NN0h3+J|(B
	return;
}

///////////////////////////////////////////////////////////////
// FUNCTION: i915_watchdog_open
// NOTE: open()$B%7%9%F%`%3!<%k(B
///////////////////////////////////////////////////////////////
static int i915_watchdog_open(struct inode *inode,struct file *file)
{
	int		wk_ret = -EBUSY;															// $BLa$jCM$O2>$G(BNG$B@_Dj(B

#ifdef DF_DEBUG_i915
	printk(KERN_INFO "i915_tco: i915_watchdog_open() have been executed.\n");		// $B%G%P%C%0%a%C%;!<%8(B
#endif


	if(test_and_set_bit(0,&G_openFlag) == 0)											// $B$^$@%*!<%W%s$7$F$$$J$$(B?
	{
		(void)nonseekable_open(inode,file);											// $B%N!<%7!<%/@_Dj(B($BLa$jCM$O>o$K(B0)
		wk_ret = 0;																		// $BLa$jCM$r@5>o$KJQ99(B
	}
	return wk_ret;
}

///////////////////////////////////////////////////////////////
// FUNCTION: i915_watchdog_release
// NOTE: close()$B%7%9%F%`%3!<%k(B
///////////////////////////////////////////////////////////////
static int i915_watchdog_release(struct inode *inode,struct file *file)
{
#ifdef DF_DEBUG_i915
	printk(KERN_INFO "i915_tco: i915_watchdog_release() have been executed.\n");	// $B%G%P%C%0%a%C%;!<%8(B
#endif


	if(G_closeReq == true)																// $B%/%m!<%:M=9pM-(B?
	{
		i915_tco_stop();																	// $B%?%$%^Dd;_(B
		G_closeReq = false;																// $B%/%m!<%:M=9p%/%j%"(B
	}

	clear_bit(0,&G_openFlag);															// $B%*!<%W%s:Q%U%i%0%/%j%"(B
#ifdef DF_DEBUG_i915
		printk(KERN_INFO "i915_tco: G_openFlag = %ld\n",G_openFlag);				// $B%G%P%C%0%a%C%;!<%8(B
		if(G_closeReq == false) printk(KERN_INFO "i915_tco: G_closeReq is ok\n");	// $B%G%P%C%0%a%C%;!<%8(B
#endif
	return 0;
}

///////////////////////////////////////////////////////////////
// FUNCTION: i915_watchdog_write
// NOTE: write()$B%7%9%F%`%3!<%k(B
///////////////////////////////////////////////////////////////
static int i915_watchdog_write(struct file *file,const char __user *data, size_t len,loff_t *ppos)
{
	char		wk_data;																		// $B%"%W%j$h$j<hF@$7$?%G!<%?(B

#ifdef DF_DEBUG_i915
	printk(KERN_INFO "i915_tco: i915_watchdog_write() have been executed.\n");		// $B%G%P%C%0%a%C%;!<%8(B
#endif

	i915_tco_reload();																		// $B%?%$%^%j%m!<%I(B
//	if(get_user(wk_data,data) == 0)															// $B%"%W%j$h$j%G!<%?<hF@!"@.8y(B?
	if(copy_from_user(&wk_data,data,1) == 0)
	{
		if(wk_data == 'V') G_closeReq = true;												// 1$BJ8;zL\$,(BV$B$J$i!"%/%m!<%:M=9pH/9T(B
	}
	return 1;																					// $B>o$K(B1$B$rJV$9(B
}

///////////////////////////////////////////////////////////////
// FUNCTION: i915_watchdog_ioctl
// NOTE: ioctl()$B%7%9%F%`%3!<%k(B
///////////////////////////////////////////////////////////////
static int i915_watchdog_ioctl(struct inode *inode,struct file *file,unsigned int cmd,unsigned long arg)
{
	int			wk_ret = 0;															// $BLa$jCM(B

#ifdef DF_DEBUG_i915
	printk(KERN_INFO "i915_tco: i915_watchdog_ioctl() have been executed.\n");	// $B%G%P%C%0%a%C%;!<%8(B
#endif

	switch(cmd)																			// $B%3%^%s%I3NG'(B
	{
	case WDIOC_SETOPTIONS:																// $B%*%W%7%g%s@_Dj(B?
		if((arg & WDIOS_DISABLECARD) != 0)												// $B%?%$%^Dd;_MW5a(B?
		{
			i915_tco_stop();																// $B%?%$%^Dd;_(B
		}
		if((arg & WDIOS_ENABLECARD) != 0)												// $B%?%$%^3+;OMW5a(B?
		{
			i915_tco_reload();															// $B%?%$%^%j%m!<%I(B
			i915_tco_start();															// $B%?%$%^3+;O(B
		}
		break;

	case WDIOC_SETTIMEOUT:																// $B%?%$%`%"%&%H;~4VJQ99MW5a(B?
		if(i915_set_timer((unsigned short)arg) == false)								// $B%?%$%`%"%&%H;~4V@_Dj!"<:GT(B?
		{
			wk_ret = -EINVAL;															// $B%(%i!<%3!<%I@_Dj(B
		}
		break;

	default:																				// $BB>$OL5;k(B
#ifdef DF_DEBUG_i915
		printk(KERN_INFO "i915_tco: i915_watchdog_ioctl() error code[%d].\n",cmd);// $B%G%P%C%0%a%C%;!<%8(B
#endif
		wk_ret = -EINVAL;																// $B%(%i!<%3!<%I@_Dj(B
		break;
	}

	return wk_ret;
}

///////////////////////////////////////////////////////////////
// FUNCTION: i915_watchdog_find
// NOTE: PCI$B%G%P%$%98!:w=hM}(B
///////////////////////////////////////////////////////////////
static bool i915_watchdog_find(void)
{
	bool		wk_ret = false;								// $BLa$jCM$r2>$G0[>o$K@_Dj(B

	G_pcidev = pci_get_device(PCI_VENDOR_ID_INTEL,
					PCI_DEVICE_ID_INTEL_ICH6_1,NULL);		// PCI$B%G%P%$%9<hF@(B
	if(G_pcidev != NULL)										// PCi$B%G%P%$%9<hF@@.8y(B?
	{
		wk_ret = true;										// $BLa$jCM$r@5>o$KJQ99(B
	}
	return wk_ret;
}

///////////////////////////////////////////////////////////////
// FUNCTION: i915_watchdog_ACPIBASE
// NOTE: ACPIBASE$B%"%I%l%9<hF@(B
///////////////////////////////////////////////////////////////
static bool i915_watchdog_ACPIBASE(void)
{
	bool						wk_ret = false;						// $BLa$jCM$O2>$G(BNG$B$K@_Dj(B
	unsigned short			wk_reg;								// $B%l%8%9%?A`:nMQ(B

	pci_read_config_word(G_pcidev,0x40,&wk_reg);					// PM_BASE$B$N2<0L(B2$B%P%$%H$h$j(BACPIBASE$B<hF@(B
	wk_reg &= 0xFFFE;												// $BITMW$J%S%C%H%/%j%"(B
	if((wk_reg != 0x0000) && (wk_reg != 0x0001))					// $B@5>o$JCM$+(B?
	{
		G_acpibase = (unsigned int)wk_reg;							// ACPIBASE$B%"%I%l%93NDj(B
		wk_ret = true;												// $BLa$jCM$r@5>o$KJQ99(B
	}
	return wk_ret;
}

///////////////////////////////////////////////////////////////
// FUNCTION: i915_watchdog_NO_REBOOT
// NOTE: NO_REBOOT$B%S%C%H%/%j%"(B
///////////////////////////////////////////////////////////////
static bool i915_watchdog_NO_REBOOT(void)
{
	bool				wk_ret = false;										// $BLa$jCM$r2>$G(BNG$B$K@_Dj(B
	unsigned int		wk_rcba;												// RCBA$B%Y!<%9%"%I%l%9(B
	int *				wk_gcs;												// GCS$B%l%8%9%?$N%"%I%l%9(B
	volatile int		wk_reg;												// $B%l%8%9%?A`:nMQ(B

	pci_read_config_dword(G_pcidev,0xF0,&wk_rcba);						// RCBA$B%Y!<%9%"%I%l%9<hF@(B
	wk_rcba &= 0xFFFFFFFE;													// $BITMW$J%S%C%H%/%j%"(B

	if(request_mem_region(wk_rcba+0x3410,4,"i915_tco") != 0)				// IO$B%a%b%j3NJ]!"@.8y(B?
	{
		wk_gcs = ioremap_nocache(wk_rcba+0x3410,4);						// $B%^%C%T%s%0(B
		wk_reg = ioread32(wk_gcs);											// GCS$B%l%8%9%?FI=P$7(B
		wk_reg &= 0xFFFFFFDF;												// NO_REBOOT$B%S%C%H%/%j%"(B
		iowrite32(wk_reg,wk_gcs);											// GCS$B%l%8%9%?$X=q9~$_(B
		wk_reg = ioread32(wk_gcs);											// $B:FEY(BGCS$B%l%8%9%?FI=P$7(B
#ifdef DF_DEBUG_i915
		printk(KERN_INFO "i915_tco: GCS = %X.\n",wk_reg);				// $B%G%P%C%0%a%C%;!<%8(B
#endif
		if((wk_reg & 0x00000002) == 0) wk_ret = true;						// NO_REBOOT$B$,%/%j%"$5$l$F$$$l$P!"La$jCM$r@5>o$KJQ99(B
		iounmap(wk_gcs);														// $B%^%C%T%s%02r=|(B
		release_mem_region(wk_rcba+0x3410,4);								// $BM=Ls$7$?(BIO$B%a%b%j$r3+J|(B
	}
	return wk_ret;
}

///////////////////////////////////////////////////////////////
// FUNCTION: i915_set_timer
// NOTE: $B%?%$%`%"%&%H;~4V@_Dj(B
///////////////////////////////////////////////////////////////
static bool i915_set_timer(unsigned short wk_time)
{
	bool		wk_ret = false;												// $BLa$jCM$O2>$G0[>o$K@_Dj(B
#ifdef DF_DEBUG_i915
	volatile unsigned short wk_reg;
#endif
	if((wk_time >= 0x02) && (wk_time <= 1023))							// $B@_Dj2DG=HO0O(B?
	{
		spin_lock(&G_spinlock);												// $B$3$N4X?t$O%7%9%F%`%3!<%k$h$j8F$P$l$k$?$aGSB>@)8f$9$k(B
		outw(wk_time,G_acpibase+0x60+0x12);								// TCO_TMR$B$K%?%$%`%"%&%H;~4V$r=q9~$_(B
		spin_unlock(&G_spinlock);											// $B%m%C%/2r=|(B
		wk_ret = true;														// $BLa$jCM$r@5>o$KJQ99(B
#ifdef DF_DEBUG_i915
		wk_reg = inw(G_acpibase+0x60+0x12);
		printk(KERN_INFO "i915_tco: TCO_TMR = %X.\n",wk_reg);			// $B%G%P%C%0%a%C%;!<%8(B
#endif

	}	
	return wk_ret;
}

///////////////////////////////////////////////////////////////
// FUNCTION: i915_tco_stop
// NOTE: $B%?%$%^Dd;_(B
///////////////////////////////////////////////////////////////
static void i915_tco_stop(void)
{
	volatile unsigned short	wk_reg;									// $B%l%8%9%?A`:nMQ(B

	spin_lock(&G_spinlock);												// $B$3$N4X?t$O%7%9%F%`%3!<%k$h$j8F$P$l$k$?$aGSB>@)8f$9$k(B
	wk_reg = inw(G_acpibase+0x60+0x08);								// TCO1_CNT$B%l%8%9%?FI=P$7(B
	wk_reg |= 0x0800;													// Halt$B%S%C%H$r%;%C%H$7$F!"%?%$%^$rDd;_(B
	outw(wk_reg,G_acpibase+0x60+0x08);									// TCO1_CNT$B$X=q9~$_(B
	spin_unlock(&G_spinlock);											// $B%m%C%/2r=|(B
	return;
}

///////////////////////////////////////////////////////////////
// FUNCTION: i915_tco_reload
// NOTE: $B%?%$%^$N%j%m!<%I(B
///////////////////////////////////////////////////////////////
static void i915_tco_reload(void)
{
	spin_lock(&G_spinlock);										// $B$3$N4X?t$O%7%9%F%`%3!<%k$h$j8F$P$l$k$?$aGSB>@)8f$9$k(B
	outw(0x0001,G_acpibase+0x60+0x00);							// TCO_RLD$B$K=q9~$_(B($B2?$r=q$$$F$b%j%m!<%I=hM}$,9T$o$l$k(B)
	spin_unlock(&G_spinlock);									// $B%m%C%/2r=|(B
	return;
}

///////////////////////////////////////////////////////////////
// FUNCTION: i915_tco_start
// NOTE: $B%?%$%^3+;O(B
///////////////////////////////////////////////////////////////
static void i915_tco_start(void)
{
	volatile unsigned short	wk_reg;									// $B%l%8%9%?A`:nMQ(B

	spin_lock(&G_spinlock);												// $B$3$N4X?t$O%7%9%F%`%3!<%k$h$j8F$P$l$k$?$aGSB>@)8f$9$k(B
	wk_reg = inw(G_acpibase+0x60+0x08);								// TCO1_CNT$B%l%8%9%?FI=P$7(B
	wk_reg &= 0xF7FF;													// Halt$B%S%C%H$r%/%j%"$7$F!"%?%$%^$r3+;O(B
	outw(wk_reg,G_acpibase+0x60+0x08);									// TCO1_CNT$B$X=q9~$_(B
	spin_unlock(&G_spinlock);											// $B%m%C%/2r=|(B
	return;
}

///////////////////////////////////////////////////////////////
// FUNCTION: i915_reboot
// NOTE: $B%j%V!<%H;~4X?t(B
///////////////////////////////////////////////////////////////
static int i915_reboot(struct notifier_block* this,unsigned long unused1, void *unused2)
{
	i915_tco_stop();				// $B%j%V!<%H$9$k$?$a!"%&%)%C%A%I%C%0%?%$%^Dd;_(B
	return NOTIFY_DONE;
}

///////////////////////////////////////////////////////////////
// FUNCTION: 
// NOTE: 
///////////////////////////////////////////////////////////////


